 #!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jun 19 15:19:29 2020
file split on Sun Oct 18 16:07:53 2020
@author: davidworrall
version 0.01: Fri Jun 19 15:19:29 2020. Set up intitial cv read
version 0.02: 2020 31 Define class structure for a country
version 0.03: 2020 08 06 Add SMA
version 0.04: 2020 09 18 finish marked-line plotting routines * non-printing vals <- 0
version 0.05: 2020 09 19 finish x-axis 7day divisions and labels
version 0.06: 2020 09 23 regional sort according to classification in WorldRegionsUpdated.csv
            complete plot generation and write to file
version 0.07 begin investigation of auto anomoly detection. found TAD (twitter ...)
see https://blog.twitter.com/engineering/en_us/a/2015/introducing-practical-and-robust-anomaly-detection-in-a-time-series.html

version 0.08 code Split code into 3 files:
    - covid19data.py  ...... included here
    - covid19class.py ...... (included in the above)
    - covid19plotting.py ... (this file) includes all of the above

databases in play:
    - deathsDB = deathsDB[1:] # drop the column headings
    - dates = datePad + columnLabels[4:]
    - countriesList = []      # collates all multiple-province country provinces into a single country entry
    - provinceDeathsDict = {} # holds all country, province and summed-province data
    - populationDataList = []       # demographic data

    - inJustDeathDB = []      # countries/provinces that are only in deaths_global.csv
    - inJustpopulationDataList = [] # countries/provinces that are only in countryPopulationData.csv
    - inBothDB = []
    - demographicDict = {}    # an amalgulm of the two .csv files data
   

   - covidInstanceDict  #  a listOf instantiated CountryNames

#some alternative color-mappings
    breakpoints = [(10,    'tab:purple'), (50,    'tab:pink'), (100,    'tab:blue'),(200,    'tab:cyan'),
                   (500,    'tab:olive'), (1000,   'green'),  (1500,   'tab:orange'),
                   (20000,  'tab:red')]      # any reduculously high number
   breakpoints = [(10, 'violet'), (50, 'indigo'), (100, 'tab:blue'),(200,'tab:green'),(500,'darkgoldenrod'),
                   (1000,''), (1500,'darkorange'),(20000,'red')]      # any reduculously high number
""" 
from covid19class import *
# ===================  covid19plotting.py - with matplotlib ====================================
def selectColor(value):
    # return a color string based on value breakpoints
    breakpoints = [(10,     'tab:blue'), 
                   (50,     'tab:cyan'),
                   (100,    'tab:green'),
                   (200,    'tab:olive'),
                   (500,    'darkgoldenrod'),
                   (1000,   'darkorange'), 
                   (1500,   'firebrick'),
                   (10000,  'tab:red')]      # any reduculously high number
    indx = 0 
    while True:
        if value <= breakpoints[indx][0]:
            return breakpoints[indx][1]
        else:
            indx += 1

import matplotlib.pyplot as plt
# =================================
covidInstanceDict = {}   # used to  store country instances for plotting etc
def graphCountries(listOfCountryNames,graphName, nrFigureCols = 1, directToFile=False):
    # draws a single graph of a list of country instances. 
    # If directToFile=True, closes the plot display after saving an image file
    title = 'Daily Deaths Reported as Covid-19 Related: '+graphName+'\n'+ dates[0][4:]+ " " + dates[0][:4] + '—' +dates[-1][4:]+ " " + dates[-1][:4]
    plt.figure (num = 1, figsize = (18.0,12.0)) 
    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths Reported", fontsize=9)
    plt.xlabel("Daily Dates (Sundays labeled)", fontsize=9)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)  # was 0.2
    plt.xticks (rotation = 45) #'vertical'
    xlabels = dates #datePad + columnLabels[4:]  # the dates. The first is Wednesday, Jan 22. Need to add 3 days to the beginning
    xmarkers = list (range(0,len(xlabels)+7, 7)) # one per week
    xlabels2 = [xlabels[datey][-6:] for datey in range(0,len(xlabels), 7)]   # omit year- take just month and day
    plt.xticks(xmarkers, xlabels2, fontsize=10)
    plt.xlim((0, len (covidInstanceDict[countryName].deathIncrements)+7)) # set to  universal temporal scale. add a week for enhanced visual spacing 

    for country in listOfCountryNames:
          instanceString = "covidInstanceDict['"+country+"'] = CountryDeaths('"+country+"')"
          exec(instanceString)                  # create the instance
#          print("attempt to execute NANs:", instanceString+'.zeros2NANs()')

          dgStr1 = "plt.plot(covidInstanceDict['"
          dgStr2 = "'].deathIncrements, label='"
          dgStr3 ="', marker='.', linewidth='0.35')"    # switch to line below for point-only graph
          dgStr3 ="', marker='H', linewidth='0.15')"
          #dgStr3 ="', marker='.', linewidth='0.0')"
          dataGraphString = dgStr1 +country+ dgStr2 +country+ dgStr3
          exec(dataGraphString)                         #  store a  created country instance in the country dictionaty

    plt.legend(title='Country/Region', loc = 'upper left', ncol=nrFigureCols, shadow = False)  # wish list + transparent=True
    datey = dates[-1].replace(dateSpacer,"").replace(" ","")    # take out the separator for the filename
    plt.savefig(imageDirectory+graphName+'_covid19_'+datey+'.png', format = 'png',
                orientation='landscape', transparent=False )
    plt.show()
    if directToFile: #  close the Figure #  default is false -- SWITCH TO TRUE TO eact
        plt.close()

imageDirectoryIndividuals = '/volumes/DRWORANGE32/COVID-19/graphImages/individualGraphs/'
from matplotlib.font_manager import FontProperties
prop = FontProperties(fname='/System/Library/Fonts/Apple Color Emoji.ttc')

# comment next line out when you want to run graphCountry()
def graphCountries(listOfCountryNames,graphName, nrFigureCols = 1, directToFile=False): pass

def graphCountry(countryName,graphName, nrFigureCols = 1, directToFile=False):
    """Graphs of a single country instance. 
       If directToFile=True, closes the plot display after saving an image file"""

    plt.figure (num = 1, figsize = (18.0,12.0))
    title = 'Daily Deaths Reported as Covid-19 Related: '+graphName+'\n'+ dates[0][4:]+ " " + dates[0][:4] + '—' +dates[-1][4:]+ " " + dates[-1][:4]
    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths Reported",   fontsize=9)
    plt.xlabel("Daily Dates (Sundays labeled)",         fontsize=9)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)  # was 0.2
    plt.xticks (rotation = 45)                      #'vertical'
    xlabels = dates                                 # The first in the DB is Wed Jan 22. Need to add 3 days to the beginning
    xmarkers = list (range(0,len(xlabels)+7, 7))    # one per week
    xlabels2 = [xlabels[datey][-6:] for datey in range(0,len(xlabels), 7)]  # omit year- take just month and day
    plt.xticks(xmarkers, xlabels2, fontsize=10)
    plt.xlim((0, len (covidInstanceDict[countryName].deathIncrements)+7))   # set to  universal temporal scalee.
                                                                            # Add a week for enhanced visual spacing 

    # instantiate the required countries - perhaps should be done en masse following class definition
    instanceString = "covidInstanceDict['"+countryName+"'] = CountryDeaths('"+countryName+"')"
    exec(instanceString)                  # create the instance

    selectedColor = selectColor (covidInstanceDict[countryName].max)   # select a color based in the map in plorColor
#    print (covidInstanceDict[countryName].max, selectedColor)
    dgStr1 = "plt.plot(covidInstanceDict['"
    dgStr2 = "'].deathIncrements, label='"
#   dgStr3 ="', marker='o', markersize='3', linewidth='0.35', color='red')"    # switch to line below for point-only graph
    dgStr3 ="', marker='H', linewidth='0.15', color='"
    dgStr4 = selectedColor + "')"
    dataGraphString = dgStr1 +countryName + dgStr2 + countryName + dgStr3 + dgStr4
#    print ('datagraphString:', dataGraphString)
    exec(dataGraphString)                         #  store a  created country instance in the country dictionaty

    plt.legend(title='Country/Region', loc = 'upper left', ncol=nrFigureCols, shadow = False)  # wish list + transparent=True
    datey = dates[-1].replace(dateSpacer,"").replace(" ","")    # take out the separator for the filename
    plt.savefig(imageDirectoryIndividuals+countryName.replace(' ','')+'_LinePlot_covid19_'+datey+'.png', format = 'png',
                orientation='landscape', transparent=False )
    plt.show()
    if directToFile: #  close the Figure #  default is false -- SWITCH TO TRUE TO eact
        plt.close()
# comment next line out when you want to run graphCountry()
#def graphCountry(countryName,graphName, nrFigureCols = 1, directToFile=False): pass 


# ==================================== Barchart  version ==============================================
def adjustY (countryName): 
    if countryName == 'China':
        return 19                   # China 	[indexes, values]: [131, 303, 294] [38, 56, 278]
    if countryName == 'Argentina':
        return 351                  #Argentina	[indexes, values]: [263, 264, 256] [484, 515, 3351]
    if countryName == 'Bolivia':
        return 165                  #Bolivia	[indexes, values]: [223, 227, 232] [92, 102, 1656]        
    if countryName == 'Ecuador':
        return 500                  #Bolivia	[indexes, values]: [223, 227, 232] [92, 102, 1656]        NB: FIX!
    if countryName == 'India':
        return 1293                 #India	[indexes, values]: [243, 240, 149] [1247, 1290, 2003]
    if countryName == 'Ireland':
        return 22                   #Ireland	[indexes, values]: [100, 92, 96] [57, 77, 220]             NB: FIX!
    if countryName == 'Kyrgyzstan':
        return 100                   #Ireland	[indexes, values]: [100, 92, 96] [57, 77, 220]    
    if countryName == 'Mexico':
        return 1789                 #Mexico	[indexes, values]: [136, 314, 260] [1092, 1217, 2789]
    if countryName == 'Pakistan':
        return 35                   #Pakistan	[indexes, values]: [152, 147, 305] [153, 178, 313]
    if countryName == 'Peru':
        return 143                  #Peru	[indexes, values]: [196, 186, 208] [593, 3887, 4143]
    if countryName == 'Sudan':
        return 27                   #Sudan	[indexes, values]: [131, 303, 294] [38, 56, 278]

def barChartCountry(countryName,graphName, nrFigureCols = 1, directToFile=False):
    """ draws a  bar chart of an individual country instance. 
    # If directToFile=True, closes the plot display after saving an image file
    """
    title = 'Daily Deaths Reported as Covid-19 Related: '+graphName+'\n'+ dates[0][4:]+ " " + dates[0][:4] + '—' +dates[-1][4:]+ " " + dates[-1][:4]
    plt.figure (num = 1, figsize = (18.0,12.0))

    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths Reported", fontsize=9)
    plt.xlabel("Daily Dates (Sundays labeled)", fontsize=9)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)  # was 0.2
    plt.xticks (rotation = 45) #'vertical'
    xlabels = dates                               # the dates. The first is Wednesday, Jan 22. Need to add 3 days to the beginning
    xmarkers = list (range(0,len(xlabels)+7, 7))  # one per week
    xlabels2 = [xlabels[datey][-6:] for datey in range(0,len(xlabels), 7)]  # omit year- take just month and day
    plt.xticks(xmarkers, xlabels2, fontsize=10)

    # instantiate the required countries - perhaps should be done en masse following class definition
    instanceString = "covidInstanceDict['"+countryName+"'] = CountryDeaths('"+countryName+"')"
    exec(instanceString)                        # create the instance

    datey = dates[-1].replace(dateSpacer,"").replace(" ","")    # take out the separator for the filename
    if countryName in outlierProvinceList:
        getMaxs = covidInstanceDict[countryName].maxVals
        print ("The Max Values and their indices:", getMaxs)
        oldy = getMaxs[-1][1]
        y = adjustY(countryName)
        x = getMaxs[-1][0]
        covidInstanceDict[countryName].deathIncrements[x] = y
        plt.annotate ('Anomoly adjustment\n was:'+str(oldy)+", now:"+str(y), xy = (x-1,y+2), xytext=(x-20,y+20), 
                  arrowprops=dict(facecolor='black', shrink=0.5))

    selectedColor = selectColor (covidInstanceDict[countryName].max)   # select a data color  based in the map in plotColor
    dgStr1 = "plt.bar(covidInstanceDict['"
    dgStr2 = "'].deathIncrements, label='"

    plt.xlim((0, len (covidInstanceDict[countryName].deathIncrements)+7)) # set to  universal temporal scale +1 week for enhanced visual spacing 
    plt.bar(xlabels, covidInstanceDict[countryName].deathIncrements,  width=1, align='center', color=selectedColor)        
    plt.savefig(imageDirectoryIndividuals+countryName.replace(' ','')+'_BarChart_covid19_'+datey+'.png', format = 'png',
                orientation='landscape', transparent=False )
    plt.show()
    if directToFile:  #  default is false -- SWITCH TO TRUE TO enact
        plt.close()

# For current testing:
#barChartCountry ('China', 'China', directToFile=True)
#barChartCountry ('Argentina', 'Argentina', directToFile=True)

# comment next line out when you want to run barChartCountry()
#def barChartCountry(countryName,graphName, nrFigureCols = 1, directToFile=False): pass 

#===================  SET UP VARIOUS COUNTRIES GROUPINGS FOR COMPOSITE PLOTTING ==========================
AmericaCentral1     = ['Mexico', 'Brazil', 'Guatemala', 'Bolivia',]

AmericaCentral2     = ['Belize', 'Cuba', 'Honduras',  'Panama', 'Nicaragua']
AmericaNorth        = ['Canada', 'Alberta','British Columbia', 'Diamond Princess', 'Grand Princess', 'Manitoba',
                       'New Brunswick', 'Newfoundland and Labrador', 'Northwest Territories', 'Nova Scotia',
                       'Ontario', 'Prince Edward Island', 'Quebec', 'Saskatchewan', 'US']
AmericaSouth        = ['Venezuela', 'Peru', 'Argentina', 'Uruguay', 'Falkland Islands (Malvinas)', 'French Guiana',
                       'Guyana','Paraguay']

AfricaNorthern1     = ['Senegal' ,'Mauritania', 'Western Sahara', 'Mali', 'Niger', 
                      'Libya','Chad',  'Eritrea','Somalia','Djibouti', 'South Sudan', 'Suriname']
AfricaNorthern2     = ['Egypt', 'Algeria', 'Morocco','Sudan',]
#AfricaCentral       = ['Ghana', 'Burkina Faso', 'Nigeria', 'Ga'zbon', 'Equatorial Guinea', 'Congo',
#                       'Zambia', 'Tanzania', 'Malawi', "Cote dIvoire", 'Togo', 'Benin', 'Burkina Faso']
AfricaCentral1      = ['Ghana',  'Zambia', 'Nigeria','Malawi',]
AfricaCentral2      = ['Burkina Faso', 'Tanzania',  'Togo', 'Benin', 'Burkina Faso', 'Cote dIvoire']  # 'Cote dIvoire', 
AfricaWest          = ['Guinea', 'Guinea-Bissau','Gambia', 'Sierra Leone', 'Liberia',  'Sao Tome and Principe']
AfricaEastern       = ['Mayotte', 'Mauritius',  'Mozambique', 'Rwanda',  'Seychelles', 'Uganda'] #'Eastern Africa',
AfricaSouthern1     = ['Namibia', 'Botswana', 'Angola','Zimbabwe','Madagascar', 'Kenya', 'Lesotho' ]
AfricaSouthern2     = ['South Africa' ]

Asia                = ['China', 'Anhui', 'Beijing', 'Chongqing', 'Fujian', 'Gansu', 'Guangdong', 'Guangxi',
                       'Guizhou', 'Hainan', 'Hebei', 'Heilongjiang', 'Henan', 'Hong Kong', 'Hubei',
                       'Hunan', 'Inner Mongolia', 'Mongolia', 'Jiangsu', 'Jiangxi', 'Jilin', 'Liaoning', 'Macau',
                       'Ningxia', 'Qinghai', 'Shaanxi', 'Shandong', 'Shanghai', 'Shanxi', 'Sichuan',
                       'Tianjin', 'Tibet', 'Xinjiang', 'Yunnan', 'Zhejiang', 'Turks and Caicos Islands',
                       'South Korea', 'Japan','Taiwan', 'Nepal', 'Burma', 'Bhutan', 'Maldives']  # NB no North Korea
AsiaNoChina         = [ 'Turks and Caicos Islands','South Korea', 'Japan','Taiwan', 'Nepal', 'Burma', 'Bhutan', 'Maldives']  # NB no North Korea
AsiaSouth           = ['Bangladesh', 'India','Pakistan', 'Sri Lanka' ]  #  'Myanmar', 
AsiaSouth1          = ['Laos', 'Thailand', 'Vietnam', 'Cambodia', 'Mongolia', 'Tajikistan']
AsiaSouth2          = ['Pakistan', 'India', 'Bangladesh', 'Sri Lanka', 'Laos', 'Thailand', 'Vietnam', 'Cambodia', 'Mongolia', 'Tajikistan' ] #'Myanmar', 
AsiaSouth3          = ['India',]
AsiaSouthEast       = ['Philippines','Malaysia','Singapore','Indonesia', 'Papua New Guinea','Timor-Leste']

Canada1             = ['Alberta', 'Northwest Territories','British Columbia',  'Manitoba', 'New Brunswick', 'Newfoundland and Labrador',
                        'Nova Scotia', 'Prince Edward Island', 'Saskatchewan'] #plus 'Canada' 'Diamond Princess', 'Grand Princess',
Canada2             = ['Ontario', 'Quebec' ] #plus 'Canada' 'Diamond Princess', 'Grand Princess',

Carribean1          = ['Anguilla', 'Antigua and Barbuda', 'Aruba', 'Bahamas', 'Barbados', 'Bermuda',
                        'Cayman Islands', 'Grenada', 'Guadeloupe', 'Haiti',]

Carribean2          = ['Jamaica', 'Martinique', 'Saint Kitts and Nevis', 'Saint Lucia', 'Saint Pierre and Miquelon',
                        'Saint Vincent and the Grenadines', 'Sint Maarten', 'Trinidad and Tobago', 'Tunisia']
Carribean3          = ['Anguilla', 'Antigua and Barbuda', 'Aruba', 'Bahamas', 'Barbados', 'Bermuda',
                        'Cayman Islands', 'Grenada', 'Guadeloupe', 'Haiti', 'Jamaica', 'Martinique', 'Saint Kitts and Nevis', 'Saint Lucia', 'Saint Pierre and Miquelon',
                        'Saint Vincent and the Grenadines', 'Sint Maarten', 'Trinidad and Tobago', 'Tunisia']

ChineseProvinces1    = ['Anhui', 'Beijing', 'Chongqing', 'Fujian', 'Gansu', 'Guangdong', 'Guangxi', 'Guizhou']
ChineseProvinces2    = ['Hainan', 'Hebei', 'Heilongjiang', 'Henan', 'Hong Kong', 'Hubei', 'Hunan', 'Inner Mongolia', 'Mongolia']
ChineseProvinces3    = ['Jiangsu', 'Jiangxi', 'Jilin', 'Liaoning', 'Macau', 'Ningxia', 'Qinghai', 'Shaanxi', ]
ChineseProvinces4    = ['Shandong', 'Shanghai', 'Shanxi', 'Sichuan', 'Tianjin', 'Tibet', 'Xinjiang', 'Yunnan', 'Zhejiang']

EuropeNorthEastern   = ['Belarus', 'Estonia', 'Iceland','Kosovo', 'Latvia', 'Lithuania', 'Poland' ,
                        'Russia', 'Kyrgyzstan', 'Slovenia']
EuropeCentralEastern1 = ['Austria', 'Bosnia and Herzegovina', 'Czechia','Hungary', 'Moldova','Romania',
                        'Serbia', 'Slovakia', 'Ukraine' ]
#EuropeCentralEastern1 = ['Austria', 'Bosnia and Herzegovina', 'Czechia','Hungary', 'Moldova','Romania',
#                        'Serbia', 'Slovakia', 'Ukraine' ]
EuropeNorthWestern1  = [ 'Denmark', 'Finland', 'Iceland', 'Norway', 'Sweden']
EuropeNorthWestern2  = [ 'Denmark', 'Finland', 'Iceland', 'Norway']
EuropeNorthWestern3  = [ 'Sweden']

EuropeSouthEastern   = ['Albania', 'Armenia', 'Azerbaijan', 'Georgia']
EuropeSouthWestern1  = ['Andorra', 'Bulgaria',  'Greece', 'Gibraltar', 'Ireland', 'Isle of Man',
                        'Liechtenstein', 'Luxembourg','Malta', 'Monaco', 'Montenegro',  'Netherlands',
                        'North Macedonia', 'Portugal', 'San Marino',  'Switzerland',  ]
EuropeSouthWestern2 = ['Andorra',  'Greece', 'Gibraltar', 'Ireland', 'Isle of Man',
                        'Liechtenstein', 'Luxembourg','Malta', 'Monaco', 'Montenegro',  'Netherlands',
                        'North Macedonia', 'Portugal', 'San Marino',  'Switzerland',  ]
EuropeSouthWestern3 = [ 'United Kingdom', 'France','Germany', 'Italy', 'Spain', 'Belgium' ]

MiddleEast1         = ['Bahrain', 'Yemen', 'Lebanon', 'Oman', 'Jordan',  'United Arab Emirates']
MiddleEast2         = ['Iran', 'Iraq', 'Saudi Arabia', 'Turkey','Israel', 'Kuwait','Kazakhstan']
MiddleEast3         = ['Qatar',  'Tajikistan', 'Syria', 'Uzbekistan', 'West Bank and Gaza']

Oceania             = ['French Polynesia', 'New Caledonia', 'Australia','Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Victoria', 'Western Australia',
                       'New Zealand']
AmericaUSA          = ['US']

AustralianStates    = ['Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Victoria', 'Western Australia']
Australasia         = ['New Zealand', 'Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Victoria', 'Western Australia']
 
noVictoria          = ['New Zealand', 'Australian Capital Territory', 'New South Wales', 'Northern Territory',
                       'Queensland', 'South Australia', 'Tasmania', 'Western Australia']


 
#===================  PLOT THE COUNTRY GROUPINGS IN COMPOSITE PLOT ==========================
#Figures now render in the Plots pane by default. 
#To make them also appear inline in the Console, uncheck "Mute Inline Plotting" under the Plots pane options menu. 

# ========  graph the composite charts =========="""  <Used to comment out the multi-graph plots>
graphCountries (AfricaNorthern1,        'Northern Africa 1 of 2',           nrFigureCols = 3, directToFile=True)
graphCountries (AfricaNorthern2,        'Northern Africa 2 of 2',           directToFile=True)
graphCountries (AfricaCentral1,         'Central Africa 1 of 2',           directToFile=True)
graphCountries (AfricaCentral2,         'Central Africa 2 of 2',           directToFile=True)
graphCountries (AfricaWest,             'West Africa',                      directToFile=True)
graphCountries (AfricaEastern,          'Eastern Africa',                   directToFile=True)
graphCountries (AfricaSouthern1,        'Southern Africa (no South Africa)',directToFile=True)
graphCountries (AfricaSouthern2,        'Southern Africa: South Africa',   directToFile=True)

graphCountries (AmericaCentral1,        'Central America 1 of 2 (Brazil & Mexico)', directToFile=True)
graphCountries (AmericaCentral2,        'Central America 2 of 2',           directToFile=True)

graphCountries (AmericaUSA,                    'United States',             directToFile=True)

graphCountries (AsiaNoChina,            'Asia-no China',                    directToFile=True)
graphCountries (AsiaSouth1,             'South Asia 1of2',                  directToFile=True)
graphCountries (AsiaSouth2,             'South Asia 2of2',                  directToFile=True)
graphCountries (AsiaSouth3,             'South Asia 2of2',                  directToFile=True)
graphCountries (AsiaSouthEast,          'South-East Asia',                  directToFile=True)

graphCountries (Australasia,            'Australasian Territories',         directToFile=True)
graphCountries (Canada1,                'Canadian Provinces-no Ontario or Quebec', directToFile=True)
graphCountries (Canada2,                'Canada: Ontario & Quebec',         directToFile=True)

graphCountries (ChineseProvinces1,       'Chinese Provinces 1 of 4',        directToFile=True)
graphCountries (ChineseProvinces2,       'Chinese Provinces 2 of 4',        directToFile=True)
graphCountries (ChineseProvinces3,       'Chinese Provinces 3 of 4',        directToFile=True)
graphCountries (ChineseProvinces4,       'Chinese Provinces 4 of 4',        directToFile=True)

graphCountries (Carribean1,              'Carribean 1 of 3',        nrFigureCols = 2, directToFile=True)
graphCountries (Carribean2,              'Carribean 2 of 3',        nrFigureCols = 2, directToFile=True)
graphCountries (Carribean3,              'Carribean 3 of 3 (All)',  nrFigureCols = 2, directToFile=True)

#graphCountries (EuropeNorthWestern1,     'North-Western Europe',           directToFile=True)
graphCountries (EuropeNorthWestern2,     'North-Western Europe (no Sweden)',directToFile=True)
graphCountries (EuropeNorthWestern3,     'North-Western Europe: Sweden)',   directToFile=True)
graphCountries (EuropeNorthEastern,      'North-Eastern Europe',             directToFile=True)
graphCountries (EuropeCentralEastern1,   'Central-Eastern Europe 1 of 1',   directToFile=True)

graphCountries (EuropeSouthWestern1,     'South-Western Europe 1 of 2', nrFigureCols = 1, directToFile=True)
graphCountries (EuropeSouthWestern2,     'South-Western Europe 2 of 2',     directToFile=True)
graphCountries (EuropeSouthWestern3,     'South-Western Europe 2 of 2',     directToFile=True)
graphCountries (EuropeSouthEastern,      'South-Eastern Europe',             directToFile=True)

graphCountries (MiddleEast1,             'Middle East (1 of 3)',             directToFile=True)
graphCountries (MiddleEast2,             'Middle East (2 of 3)',             directToFile=True)
graphCountries (MiddleEast3,             'Middle East (3 of 3)',             directToFile=True)
#graphCountries (Asia,                   'Asia',                            directToFile=True)
graphCountries (Oceania,                 'Oceania',                          directToFile=True)

#graphCountries (NorthAmerica, 'North America', directToFile=True)
#graphCountries (AustralianStates,       'Australian States',               directToFile=True)

graphCountries (noVictoria,              'Australia (no Victoria)',          directToFile=True)

# ========  END OF   graph the composite charts ==========
#  </Used to comment out the multi-graph plots>"""

# =====================================================================================================
# ====================================  INDIVIDUAL COUNTRY GRAPHINGS ==================================

# a list not for individual plotting (mostly becasue they are v small in number): easily changed!
forfixingList =  ['Argentina', 'Bolivia', 'Chile', 'Ecuador', 'India', 'Ireland', 'Kyrgyzstan', 'Mexico', 'Peru']
sparceList = ['Albania', 'Tunisia','Zambia']

exclusionsList = ['Andorra', 'Angola', 'Anguilla', 'Antigua and Barbuda', 'Aruba', 
                  'Barbadas', 'Belize', 'Benin', 'Bermuda', 'Bhutan', 'Bonaire Sint Eustatius and Saba', 
                  'Botswana', 'British Virgin Islands', 'Brunei', 'Burkina Faso', 'Burundi',
                  'Cabo Verde', 'Cambodia', 'Cape Verde', 'Cayman Islands', 'Central African Republic', 
                  'Chad', 'Comoros', 'Cote dIvoire', 'Cuba', 'Curacao', 'Cyprus', 
                  'Djibouti', 'Dominica', 'Equatorial Guinea', 'Eritrea', 'Estonia', 'Eswatini',
                  'Falkland Islands (Malvinas)', 'Faroe Islands', 'Fiji', 'French Guiana', 'French Polynesia', 
                  'Gabon', 'Gibraltar', 'Greenland', 'Grenada', 'Guadeloupe', 'Guinea-Bissau', 
                  'Guinea', 'Guyana', 'Haiti', 'HongKong', 'Iceland', 'Isle of Man', 'Jamaica', 
                  'Laos', 'Latvia', 'Lesotho', 'Liberia', 'Liechtenstein', 'Lithuania', 'Luxembourg', 
                  'Macau', 'Madagascar', 'Malawi', 'Malaysia', 'Maldives', 'Mali', 'Malta', 'Marshall Islands', 'Martinique',
                  'Mauritania', 'Mauritius', 'Mayotte', 'Monaco', 'Mongolia', 'Montenegro', 'Montserrat', 
                  'Mozambique', 'Namibia', 'New Caledonia', 'New Zealand', 'Nicaragua', 'Niger', 
                  'Papua New Guinea', 'Qatar', 'Reunion', 'Rwanda', 'Saint Barthelemy', 
                  'Saint Kitts and Nevis', 'Saint Lucia', 'Saint Pierre and Miquelon', 'San Marino', 
                  'Sao Tome and Principe', 'Senegal', 'Seychelles', 'Sierra Leone', 'Singapore', 
                  'Sint Maarten', 'Slovakia', 'Slovenia', 'Solomon Islands', 'Somalia', 'South Sudan', 
                  'Sri Lanka', 'Suriname', 'Syria', 'Taiwan', 'Tajikistan', 'Tanzania', 'Thailand', 
                  'Timor-Leste', 'Togo', 'Trinidad and Tobago', 'Turk sand Caicos Islands', 
                  'Uganda', 'Uruguay', 'Uzbekistan', 'Vanuatu', 'Vatican', 'Vietnam', 'Western Sahara', 'Zimbabwe']                                          


"""# this tests if the countries in the exclusionList are in the demographicDict.keys(), used below
for country in exclusionsList:
    if country not in demographicDict.keys():
        print (country)
print ("FIN")
"""

# ========  graph major individual countries/regions =====================
#def graphCountry(countryName,graphName, nrFigureCols = 1, directToFile=False):

#    Amalgamate info in the virus data bases with the demographic/population databases: cut to another data module

instantiatedCountryStats = [] 
for country in covidInstanceDict.keys():     #create a list of lists
    instantiatedCountryStats.append([
    covidInstanceDict[country].countryName, 
    covidInstanceDict[country].latitude, 
    covidInstanceDict[country].longitude, 
    covidInstanceDict[country].area, 
    covidInstanceDict[country].population, 
    covidInstanceDict[country].max, 
    covidInstanceDict[country].mean, 
    covidInstanceDict[country].median,  
    covidInstanceDict[country].sortedIndices[-2], 
    covidInstanceDict[country].deathIncrements[covidInstanceDict[country].sortedIndices[-2]], 
    covidInstanceDict[country].sortedIndices[-1], 
    covidInstanceDict[country].deathIncrements[covidInstanceDict[country].sortedIndices[-1]]
    ])

directory =         "/Users/davidworrall/Documents/RESEARCH/COVID-19/dataSets/"
directory =         "/volumes/DRWORANGE32/COVID-19/dataSets/"
imageDirectory =    "/volumes/DRWORANGE32/COVID-19/graphImages/deaths_global/"
fname3 =            "instantiatedCountryStats.csv" # "us-counties20200516.csv"
columnLabels =      ['Country','Latitude','Longitude','Area','Population','MaxNr','Mean','Median','AllButIndx',
                     'AllButVal', 'indxOfBiggest', 'BiggestVal']
#columnLabels = ''.join (columnLabels)
import csv
with open(directory + fname3, '+w', newline='') as myfile:
        writer = csv.writer(myfile, delimiter=',')
        writer.writerow(columnLabels)
        writer.writerows(instantiatedCountryStats)
myfile.close()

for kee in covidInstanceDict2.keys():
    print (kee)
    barChartCountry(kee, kee, nrFigureCols = 1, directToFile=True)


    
   #  =======  collate statistics from the combined database - for meta-analysis to a .csv file ===============
"""
for kee in  demographicDict.keys():
    first direct from the databases: 
    lat, long, area, population, popden, median_age
    (minval_1>0, addr1), (minval12,addr2), (maxval_2, addr2), totalNrDeaths, density=pop/areaCol

plt.title("Daily Covid-19 Deaths\nfor the period "+columnLabels[4]+ '–' +columnLabels[-1], fontsize=18)
plt.ylabel("Number of Confirmed Deaths", fontsize=12)
plt.legend(title='Country/Region')
plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.25)

#  ------- WORKING ---- UNFINISHED ------------------------------------------
def graphAllProvinces(listOfCountryNames,graphName, nrFigureCols = 1, directToFile=False):
    # draws a single graph of all provinces in the ??? dict (a separate instantiation dictionary). 
    # If directToFile=True, closes the plot display after saving an image file
    title = 'Daily Deaths Reported as Covid-19 Related: '+graphName+'\n'+ dates[0][4:]+ " " + dates[0][:4] + '—' +dates[-1][4:]+ " " + dates[-1][:4]
    plt.figure (num = 1, figsize = (18.0,12.0)) 
    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths Reported", fontsize=9)
    plt.xlabel("Daily Dates (Sundays labeled)", fontsize=9)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)  # was 0.2
    plt.xticks (rotation = 45) #'vertical'
    xlabels = dates #datePad + columnLabels[4:]  # the dates. The first is Wednesday, Jan 22. Need to add 3 days to the beginning
    xmarkers = list (range(0,len(xlabels)+7, 7)) # one per week
    xlabels2 = [xlabels[datey][-6:] for datey in range(0,len(xlabels), 7)]  # omot year- take just month and day
    plt.xticks(xmarkers, xlabels2, fontsize=10)

    for country in listOfCountryNames:
          instanceString = "covidInstanceDict['"+country+"'] = CountryDeaths('"+country+"')"
          exec(instanceString)                  # create the instance
#          print("attempt to execute NANs:", instanceString+'.zeros2NANs()')

          dgStr1 = "plt.plot(covidInstanceDict['"
          dgStr2 = "'].deathIncrements, label='"
          dgStr3 ="', marker='.', linewidth='0.35')"    # switch to line below for point-only graph
          dgStr3 ="', marker='H', linewidth='0.15')"
          #dgStr3 ="', marker='.', linewidth='0.0')"
          dataGraphString = dgStr1 +country+ dgStr2 +country+ dgStr3
          exec(dataGraphString)                         #  store a  created country instance in the country dictionaty

    plt.legend(title='Country/Region', loc = 'upper left', ncol=nrFigureCols, shadow = False)  # wish list + transparent=True
    datey = dates[-1].replace(dateSpacer,"").replace(" ","")    # take out the separator for the filename
    plt.savefig(imageDirectory+graphName+'_covid19_'+datey+'.png', format = 'png',
                orientation='landscape', transparent=False )
    plt.show()
    if directToFile: #  close the Figure #  default is false -- SWITCH TO TRUE TO eact
        plt.close()

def graphCountries(listOfCount ryNames,graphName, nrFigureCols = 1, directToFile=False):
    # draws a single graph of a list of country instances. 
    # If directToFile=True, closes the plot display after saving an image file
    title = 'Daily Deaths Reported as Covid-19 Related: '+graphName+'\n'+ dates[0][4:]+ " " + dates[0][:4] + '—' +dates[-1][4:]+ " " + dates[-1][:4]
    plt.figure (num = 1, figsize = (18.0,12.0)) 
    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths Reported", fontsize=9)
    plt.xlabel("Daily Dates (Sundays labeled)", fontsize=9)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)  # was 0.2
    plt.xticks (rotation = 45) #'vertical'
    xlabels = dates #datePad + columnLabels[4:]  # the dates. The first is Wednesday, Jan 22. Need to add 3 days to the beginning
    xmarkers = list (range(0,len(xlabels)+7, 7)) # one per week
    xlabels2 = [xlabels[datey][-6:] for datey in range(0,len(xlabels), 7)]  # omot year- take just month and day
    plt.xticks(xmarkers, xlabels2, fontsize=10)

    for country in listOfCountryNames:
          instanceString = "covidInstanceDict['"+country+"'] = CountryDeaths('"+country+"')"
          exec(instanceString)                  # create the instance
#          print("attempt to execute NANs:", instanceString+'.zeros2NANs()')

          dgStr1 = "plt.plot(covidInstanceDict['"
          dgStr2 = "'].deathIncrements, label='"
          dgStr3 ="', marker='.', linewidth='0.35')"    # switch to line below for point-only graph
          dgStr3 ="', marker='H', linewidth='0.15')"
          #dgStr3 ="', marker='.', linewidth='0.0')"
          dataGraphString = dgStr1 +country+ dgStr2 +country+ dgStr3
          exec(dataGraphString)                         #  store a  created country instance in the country dictionaty

    plt.legend(title='Country/Region', loc = 'upper left', ncol=nrFigureCols, shadow = False)  # wish list + transparent=True
    datey = dates[-1].replace(dateSpacer,"").replace(" ","")    # take out the separator for the filename
    plt.savefig(imageDirectory+graphName+'_covid19_'+datey+'.png', format = 'png',
                orientation='landscape', transparent=False )
    plt.show()
    if directToFile: #  close the Figure #  default is false -- SWITCH TO TRUE TO eact
        plt.close()

"""